'' =================================================================================================
''
''   File....... jm_ina219b_demo.spin
''   Purpose.... 
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (c) 2014 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 
''   Updated.... 06 APR 2014
''
'' =================================================================================================


con { timing }

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                          ' use 5MHz crystal

  CLK_FREQ = ((_clkmode - xtal1) >> 6) * _xinfreq               ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                   ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                               ' ticks in 1us


con { io pins }

  RX1 = 31                                                      ' programming / terminal
  TX1 = 30
  
  SDA = 29                                                      ' eeprom / i2c
  SCL = 28


con { pst formatting }

   #1, HOME, GOTOXY, #8, BKSP, TAB, LF, CLREOL, CLRDN, CR
  #14, GOTOX, GOTOY, CLS


con { calibration }

  ' Calibration values setup for
  ' -- 32.000v
  ' --  0.320v shunt max
  ' --  0.100  Ohm shunt (Adafruit pcb)
  ' --  0.250A max expected                                     ' for LED testing
  ' --  10uA LSB

  CAL    = $A000
  ISCALE =  1000                                                ' convert uA to mA
  ILSB   =    10                                                ' LSB value for current (uA)
  WLSB   =    20 * ILSB                                         ' LSB value for power

  
obj

  term : "fullduplexserial64"                                   ' serial for terminal
  imon : "jm_ina219b"                                           ' current monitor
 

var


dat

  Screen                byte    "==========================", CR
                        byte    "INA219b Demo (LED Measure)  ", CR
                        byte    "==========================", CR
                        byte    CR
                        byte    "Present......... ", CR
                        byte    CR
                        byte    "Shunt Voltage... ", CR
                        byte    "Bus Voltage..... ", CR
                        byte    "Current......... ", CR
                        byte    "Power........... ", CR
                        byte    0
                        

pub main | cfg, t

  term.start(RX1, TX1, %0000, 115_200)
  imon.start(14, 15, %00)                                       ' tested on Propeller BoE
  pause(10)

  { open PST, enable, then press key }
  
  term.rxflush
  term.rx     

  term.tx(CLS)                                                  ' setup output screen
  term.str(@Screen)

  if (imon.present)
    move_to(17, 4)
    term.str(string("Yes - Calibrating"))
    pause(100)
    imon.calibrate(CAL)
    move_to(20, 4)
    term.tx(CLREOL)

    ' max bus volts = 32v
    ' max shunt volts = 0.32
    ' 12-bit resolution for bus
    ' 1x sampling of shunt
    ' shunt and bus continous sampling
    
    cfg := imon#BRNG_32 | imon#PG_320 | imon#BADC_012B | imon#SADC_001X | imon#MD_SBC
    imon.configure(cfg)

    t := cnt
    repeat
      waitcnt(t += (100 * MS_001))                              ' give time to sample
      report                                                    ' show current measurement

  else
    move_to(17, 4)
    term.str(string("No"))      
    repeat
      waitcnt(0)
                                

pub report | v, i, w

  move_to(17, 6)
  v := imon.shunt_volts
  term.rjdec(v  / 1000, 2, " ")
  term.tx(".")
  term.rjdec(v // 1000, 3, "0")

  move_to(17, 7)
  v := imon.bus_volts
  term.rjdec(v  / 1000, 2, " ")
  term.tx(".")
  term.rjdec(v // 1000, 3, "0")

  move_to(17, 8)
  i := imon.current 
  term.rjdec(i  / (ISCALE/ILSB), 2, " ")
  term.tx(".")
  term.rjdec(i // (ISCALE/ILSB), 3, "0")
  term.str(string(" mA"))

  move_to(17, 9)
  w := imon.power * WLSB
  term.rjdec(w  / 1000, 2, " ")
  term.tx(".")
  term.rjdec(w // 1000, 3, "0")
  term.str(string(" mW"))
   

pub move_to(x, y)

'' Move PST cursor to x,y

  term.tx(GOTOXY)
  term.tx(x)
  term.tx(y)

    
con

  { ------------- }
  {  B A S I C S  }
  { ------------- }


pub pause(ms) | t

'' Delay program in milliseconds

  if (ms < 1)                                                   ' delay must be > 0
    return
  else
    t := cnt - 1776                                             ' sync with system counter
    repeat ms                                                   ' run delay
      waitcnt(t += MS_001)
    

pub high(pin)

'' Makes pin output and high

  outa[pin] := 1
  dira[pin] := 1


pub low(pin)

'' Makes pin output and low

  outa[pin] := 0
  dira[pin] := 1


pub toggle(pin)

'' Toggles pin state

  !outa[pin]
  dira[pin] := 1


pub input(pin)

'' Makes pin input and returns current state

  dira[pin] := 0

  return ina[pin]


dat

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}  